"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Router_1 = __importDefault(require("./Router"));
const HttpError_1 = require("../HttpError");
const Util_1 = require("../Util");
/** Routing Class
 *
 * Top Level Wrapper around Router class that
 * handles errors.
 */
class Routing extends Router_1.default {
    /** Routing Constructor
     *
     */
    constructor() {
        super("");
        this._errors.set((err, ctx) => {
            const status = isNaN(err.status) ? 500 : Number(err.status);
            const content = (ctx.request.headers.get("Content-Type") || "").toLocaleLowerCase();
            ctx.status(status);
            if (ctx.expectsRender() || content.includes("html")) {
                try {
                    ctx.render({
                        head: {
                            title: err.message
                        },
                        body: {
                            error: `${status}: ${err.message}`
                        }
                    });
                }
                catch (_) {
                    ctx.html(`<html><head><title>${err.message}</title><style>#error{color: red}</head><body><h1 id="error">${status}: ${err.message}</h1></body></html>`);
                }
            }
            else if (content.includes("json")) {
                ctx.json(err);
            }
            else {
                ctx.write(err.message);
            }
            ctx.end();
        });
    }
    /** Routing Handler Wrappper
     *
     *
     * @param {Context} context
     * @returns {Promise<Response>}
     */
    async route(context) {
        const start = Date.now();
        await this.handle(context);
        //Logging step if Node.js
        if ((0, Util_1.inNodeEnvironment)()) {
            const time = Date.now() - start;
            const status = context.response.status;
            const statusText = (0, HttpError_1.getMessage)(status) || "Unknown Error";
            console.log(`${context.method} ${context.path} ${status} ${statusText} (${time})`);
        }
        return await context.flush();
    }
    /** Handle Routing Override
     *
     * @param {Context} context
     * @returns {Promise<void>}
     */
    async handle(context) {
        try {
            for (const { name, layer } of this._methods) {
                if (name === "MIDDLEWARE") {
                    await layer.handle(context);
                    if (context.response.commited())
                        return;
                }
                else if (name === context.method || name === "ALL") {
                    await layer.handle(context);
                    if (context.response.commited())
                        return;
                }
            }
            throw 404;
        }
        catch (err) {
            this._errors.handle(err, context);
        }
    }
}
exports.default = Routing;
