"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Util_1 = require("./Util");
const Head_1 = __importDefault(require("./Head"));
const Util_2 = require("../../Util");
const LOCK_STRING = "a, button, input, textarea, select";
/** Get Dialog Level Settings
 *
 * @param {string} level
 * @returns {Object}
 */
function getLevelSettings(level) {
    switch (level) {
        case "error":
            return {
                background: "#FFE5E5",
                foreground: "#D60007",
                title: "Error!"
            };
        case "warning":
            return {
                background: "#FFFEE5",
                foreground: "#BD5800",
                title: "Warning!"
            };
        case "info":
            return {
                background: "#E3EEFD",
                foreground: "#0260ED"
            };
        default:
            throw new Error("Unknown Settings Level!");
    }
}
/** Front End Render Environment
 *
 */
class RenderEnvironment {
    /** Render Environment Constructor
     *
     */
    constructor() {
        this._routing = false;
        this._head = new Head_1.default();
        this._headHash = 0;
        this._bodyHash = 0;
        this._events = [];
    }
    /** Main Route Handler
     *
     * Returns value if there is an anchor to scroll too.
     *
     * @param {FormData} body
     * @returns {Promise<string|undefined>}
     */
    async handler(url = window.location.href, opts) {
        this._routing = true;
        this.lock();
        const { anchor, path } = (0, Util_1.getRouteInfo)(url);
        try {
            const data = await this.fetch(path, opts);
            if (data.redirect) {
                return this.handler(data.redirect);
            }
            await this.update(data);
            window.history.pushState({}, "", url);
        }
        catch (e) {
            this.error(e);
            //Possible rerouting??
        }
        this._routing = false;
        this.unlock();
        if (this._delay) {
            const { url, opts } = this._delay;
            this._delay = undefined;
            return this.handler(url, opts);
        }
        return anchor;
    }
    /** Route to locale page.
     *
     * @param {string|URL} url
     * @param {FetchOptions} opts
     */
    async route(url, opts) {
        if (!this._routing) {
            this.clear();
            const anchor = await this.handler(url, opts);
            if (anchor)
                this.scroll(anchor);
        }
        else {
            this._delay = { url, opts };
        }
    }
    /** Scroll to Element
     *
     * @param {string} id
     */
    scroll(id) {
        if (id) {
            const element = document.getElementById(id);
            if (element) {
                element.scrollIntoView();
            }
        }
    }
    /** Open External Link
     *
     * @param {string|URL} href
     */
    link(href) {
        window.open(href, '_blank', 'noopener,noreferrer');
    }
    /** Run Script
     *
     * @param script
     */
    run(script) {
        new Function("env", script)(this);
    }
    /** Add Event
     *
     * @param update
     */
    event(type, listener) {
        if (typeof type !== "string")
            throw new TypeError("Unknown event!");
        if (typeof listener !== "function")
            throw new TypeError("Unknown listener!");
        this._events.push({ type, listener });
        window.addEventListener(type, listener);
    }
    /** Update Environment
     *
     * @param {RenderUpdate} update
     */
    async update(update) {
        if (update.head) {
            await this.updateHead(update.head);
        }
        if (update.body) {
            this.updateBody(update.body);
        }
    }
    ////////////// Private Update Methods //////////////
    /** Update Head
     *
     * @param {HeadUpdate} update
     */
    async updateHead(update) {
        const hash = (0, Util_1.hashObject)(update);
        if (hash === this._headHash) {
            return console.info("Head didn't change!");
        }
        await this._head.update(update);
        this._headHash = hash;
    }
    /** Update Body
     *
     */
    updateBody(update) {
        const hash = (0, Util_1.hashObject)(update);
        const scripts = [];
        if (hash === this._bodyHash) {
            return console.info("Body didn't change!");
        }
        for (const id in update) {
            const element = document.getElementById(id);
            if (element) {
                this.render(element, update[id]);
                const match = update[id].match(/<script.*?>.*?<\/script.*?>/gi);
                if (match) {
                    scripts.push(...match);
                }
            }
            else {
                console.warn(`Unable to find element with id '${id}'!`);
            }
        }
        this.scripts(scripts);
        this._bodyHash = hash;
    }
    /** Run Scripts
     *
     * @param {RegExpMatchArray} update
     */
    scripts(update) {
        for (let script of update) {
            this.run(script.replace(/^<script.*?>(.*?)<\/script.*?>$/gi, "$1"));
        }
    }
    /** Clear Events
     *
     */
    clear() {
        while (this._events.length > 0) {
            const { type, listener } = this._events.pop();
            window.removeEventListener(type, listener);
        }
    }
    /** Modal Dialog
     *
     * @param {string} level
     * @param {string} message
     */
    dialog(level, message) {
        const { background, foreground, title } = getLevelSettings(level);
        const dialog = document.createElement("dialog");
        dialog.style.position = "absolute";
        dialog.style.width = "100%";
        dialog.style.height = "100%";
        dialog.style.top = "0";
        dialog.style.left = "0";
        dialog.style.display = "flex";
        dialog.style.alignItems = "center";
        dialog.style.justifyContent = "center";
        dialog.style.background = "rgba(0,0,0,0.5)";
        dialog.addEventListener("click", () => {
            document.body.removeChild(dialog);
        });
        const modal = document.createElement("div");
        modal.style.backgroundColor = background;
        modal.style.padding = "10px";
        modal.style.borderRadius = "5px";
        modal.style.border = `solid ${foreground}`;
        modal.addEventListener("click", (event) => {
            event.stopPropagation();
        });
        if (title) {
            const header = document.createElement("h1");
            header.style.color = foreground;
            header.style.fontWeight = "bold";
            header.style.textAlign = "center";
            header.style.margin = "0";
            header.textContent = title;
            modal.appendChild(header);
        }
        const body = document.createElement("p");
        body.style.color = foreground;
        body.style.fontWeight = "bold";
        body.style.textAlign = "center";
        body.textContent = String(message);
        modal.appendChild(body);
        const close = document.createElement("button");
        close.style.display = "block";
        close.style.margin = "0 auto";
        close.textContent = "Ok";
        modal.appendChild(close);
        close.addEventListener("click", () => {
            document.body.removeChild(dialog);
        });
        dialog.appendChild(modal);
        dialog.open = true;
        document.body.appendChild(dialog);
    }
    ////////////// Static FunctionS //////////////
    /** Assign Render Content to Target
     *
     * @param {HTMLElement} target
     * @param {RenderContent} content
     */
    render(target, content) {
        if (target.value) {
            target.value = content;
        }
        else {
            target.innerHTML = content;
        }
    }
    /** Fetch Wrapper
     *
     * @param {stirng|URL} url
     * @param {FetchOptions} opts
     * @returns {Promise<RenderUpdate>}
     */
    async fetch(url, opts = {}) {
        if (opts.headers === undefined)
            opts.headers = {};
        opts.headers[Util_2.HEADER_KEY] = Util_2.HEADER_VALUE;
        const response = await fetch(url, opts);
        if (response.headers.get(Util_2.HEADER_KEY) !== Util_2.HEADER_VALUE) {
            throw new Error("Did not recieve an update response!");
        }
        else if (response.headers.get("Content-Type") !== "application/json") {
            throw new Error("Did not recieve JSON response!");
        }
        const data = await response.json();
        if (data.redirect === undefined && data.head === undefined && data.body === undefined && data.update === undefined) {
            throw new Error("Recieved either an empty or invalid response!");
        }
        return data;
    }
    /** Display Error
     *
     * @param {any} value
     */
    error(value) {
        this.dialog("error", value.message || String(value));
        console.error(value);
    }
    /** Display Warning
     *
     * @param {any} value
     */
    warn(value) {
        this.dialog("warning", value);
        console.warn(value);
    }
    /** Display Warning
     *
     * @param {any} value
     */
    alert(value) {
        this.dialog("info", value);
    }
    /** Display Warning
     *
     * @param {any} value
     */
    info(value) {
        this.dialog("info", value);
        console.info(value);
    }
    lock(cursor = "wait") {
        document.body.style.cursor = cursor;
        document.querySelectorAll(LOCK_STRING).forEach(element => {
            element.ariaDisabled = "true";
            element.toggleAttribute("disabled", true);
            element.style.cursor = cursor;
        });
    }
    unlock() {
        document.body.style.cursor = "";
        document.querySelectorAll(LOCK_STRING).forEach(element => {
            element.removeAttribute("aria-disabled");
            element.toggleAttribute("disabled", false);
            element.style.cursor = "";
        });
    }
}
exports.default = RenderEnvironment;
